import unittest
from tests.test_commands import *
from tests.test_inputs import *
from tests.test_misc import *
from tests.test_outputs import *
from tests.test_shapes import *
from tests.test_defaults import *
from tests.test_config import *
from tests.test_formatting import *
from tests.test_datasets import *
from tests.test_mycollections import *

if __name__ == '__main__':
    unittest.main()
